"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var glob_1 = require("glob");
var mkdirp_1 = require("mkdirp");
var lodash_1 = require("lodash");
var path_1 = __importDefault(require("path"));
var fs_1 = __importDefault(require("fs"));
var assert_1 = __importDefault(require("assert"));
var CodeGenApi = /** @class */ (function () {
    function CodeGenApi(opts) {
        var configuration = opts.configuration, absFilename = opts.absFilename, renderer = opts.renderer, inputExtension = opts.inputExtension, outputExtension = opts.outputExtension;
        this.configuration = configuration;
        this.templateFilenameAbs = opts.absFilename;
        this.templateDirnameAbs = path_1.default.dirname(this.templateFilenameAbs);
        this.templateDirname = path_1.default.relative(configuration.rootDir, this.templateDirnameAbs);
        this.templateFilename = path_1.default.relative(configuration.rootDir, this.templateFilenameAbs);
        this.emitFilenameAbs = this.templateFilenameAbs.replace(new RegExp(lodash_1.escapeRegExp(inputExtension) + "$"), outputExtension);
        this.emitFilename = path_1.default.relative(configuration.outDir, this.emitFilenameAbs);
        this.templateExtension = inputExtension;
        this.emitExtension = outputExtension;
    }
    Object.defineProperty(CodeGenApi.prototype, "templateSourceText", {
        get: function () {
            if (!this._templateSourceText) {
                this._templateSourceText = fs_1.default.readFileSync(this.templateFilenameAbs, 'utf8');
            }
            return this._templateSourceText;
        },
        enumerable: true,
        configurable: true
    });
    CodeGenApi.prototype.emit = function (filename, content) {
        if (filename == null) {
            filename = this.emitFilenameAbs;
        }
        if (!path_1.default.isAbsolute(filename)) {
            filename = path_1.default.join(this.templateDirnameAbs, filename);
        }
        console.log("Emitting: " + path_1.default.relative(this.configuration.cwdAbs, filename));
        mkdirp_1.sync(path_1.default.dirname(filename));
        fs_1.default.writeFileSync(filename, content);
    };
    return CodeGenApi;
}());
exports.CodeGenApi = CodeGenApi;
var defaultConfiguration = {
    cwd: process.cwd(),
    rootDir: 'src',
    outDir: 'src',
    include: ["src/**"],
    exclude: new Array(),
    /** Modules to preload TODO implement */
    require: [],
    extensions: {
        '.generate.ts': ['typescript', '.ts']
    },
    renderers: {
        typescript: function (api) {
            // TODO Load ts-node on-demand
            // TODO read settings from tsconfig
            require('ts-node').register({
                transpileOnly: true
            });
            var exports = require(api.templateFilenameAbs);
            var fn;
            if (exports && typeof exports.default === 'function') {
                fn = function () {
                    var args = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        args[_i] = arguments[_i];
                    }
                    return exports.default.apply(exports, __spread(args));
                };
            }
            else {
                fn = exports;
                assert_1.default(typeof fn === 'function');
            }
            var returnValue = fn(api);
            if (typeof returnValue === 'string') {
                api.emit(api.emitFilenameAbs, returnValue);
            }
        }
    }
};
function main() {
    var e_1, _a;
    // TODO this should be the path of the config file
    var cwdAbs = path_1.default.resolve(process.cwd(), defaultConfiguration.cwd);
    var rootDir = defaultConfiguration.rootDir, outDir = defaultConfiguration.outDir;
    var configuration = __assign({}, defaultConfiguration, { cwdAbs: cwdAbs, rootDirAbs: path_1.default.resolve(cwdAbs, rootDir), outDirAbs: path_1.default.resolve(cwdAbs, outDir) });
    var inputExtensions = Object.keys(configuration.extensions);
    var matches = lodash_1.uniq(lodash_1.flatten(lodash_1.map(configuration.include, function (i) {
        return glob_1.sync(i, {
            absolute: true,
            cwd: cwdAbs,
            dot: true,
            ignore: configuration.exclude,
            nodir: true
        });
    }))).map(function (v) {
        var e_2, _a;
        try {
            // Filter by files that match an extension we care about
            for (var inputExtensions_1 = __values(inputExtensions), inputExtensions_1_1 = inputExtensions_1.next(); !inputExtensions_1_1.done; inputExtensions_1_1 = inputExtensions_1.next()) {
                var ext = inputExtensions_1_1.value;
                if (v.slice(-ext.length) === ext) {
                    return {
                        file: v,
                        extension: ext
                    };
                }
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (inputExtensions_1_1 && !inputExtensions_1_1.done && (_a = inputExtensions_1.return)) _a.call(inputExtensions_1);
            }
            finally { if (e_2) throw e_2.error; }
        }
        return undefined;
    }).filter(lodash_1.identity);
    try {
        for (var matches_1 = __values(matches), matches_1_1 = matches_1.next(); !matches_1_1.done; matches_1_1 = matches_1.next()) {
            var match = matches_1_1.value;
            var _b = match, extension = _b.extension, file = _b.file;
            var _c = __read(configuration.extensions[extension], 2), rendererName = _c[0], outputExtension = _c[1];
            var renderer = configuration.renderers[rendererName];
            var api = new CodeGenApi({
                absFilename: file,
                inputExtension: extension,
                configuration: configuration,
                outputExtension: outputExtension,
                renderer: renderer,
            });
            console.log("Rendering: " + path_1.default.relative(configuration.cwdAbs, api.templateFilenameAbs));
            renderer(api);
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (matches_1_1 && !matches_1_1.done && (_a = matches_1.return)) _a.call(matches_1);
        }
        finally { if (e_1) throw e_1.error; }
    }
    return 0;
}
function start(isCliInvocation) {
    if (isCliInvocation) {
        process.exit(main());
    }
}
exports.start = start;
//# sourceMappingURL=main.js.map